<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 09/10/2015
 * Time: 10:11
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use Factory\Repository as RepositoryFactory;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Repository;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Project;
use Factory\ZhUserAllocation as ZhUserAllocationFactory;
use Factory\Product as ProductFactory;
use Zeedhi\DevOps\Util\Environment;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Configurationmanager\Entities as ConfigurationEntities;
use Zeedhi\DevOps\Model\Profile\Entities as ProfileEntities;
use Zeedhi\DevOps\Model\Devops\Entities as DevOpsEntities;
use Zeedhi\DevOps\Model\Devops\Repositories as DevOpsRepositories;
use Zeedhi\DevOps\Model\Profile\Repositories as ProfileRepositories;
use Zeedhi\DevOps\Model\Configurationmanager\Repositories as ConfigurationRepositories;
use Zeedhi\DevOps\Model\Projectmanagement\Entities as PmEntities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories as PmRepositories;

class Product {

    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var RepositoryFactory $entityManager */
    protected $repositoryFactory;
    /** @var ProductFactory $productFactory */
    protected $productFactory;
    /** @var ConfigurationRepositories\Product $productRepository */
    protected $productRepository;
    /** @var ConfigurationRepositories\ProductFiles $productFilesRepository */
    protected $productFilesRepository;
    /** @var DevOpsRepositories\ProductFilesMnemonic $productFilesMnemonicRepository */
    protected $productFilesMnemonicRepository;
    /** @var ConfigurationRepositories\SubreleaseProductFiles $subreleaseProductFilesRepository */
    protected $subreleaseProductFilesRepository;
    /** @var ProfileRepositories\ZhUser $userRepository */
    protected $userRepository;
    /** @var DevOpsRepositories\ZhUserAllocation $userAllocationRepository */
    protected $userAllocationRepository;
    /** @var PmRepositories\Project $projectRepository */
    protected $projectRepository;
    /** @var Environment  */
    private $environment;

    /**
     * @param EntityManager $entityManager
     * @param Environment $environment
     * @param RepositoryFactory $repositoryFactory
     * @param ZhUserAllocationFactory $zhUserAllocationFactory
     * @param ProductFactory $productFactory
     */
    public function __construct(EntityManager $entityManager, Environment $environment, RepositoryFactory $repositoryFactory, ZhUserAllocationFactory $zhUserAllocationFactory,
                                ProductFactory $productFactory) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->repositoryFactory = $repositoryFactory;
        $this->zhUserAllocationFactory = $zhUserAllocationFactory;
        $this->productFactory = $productFactory;

        //DevOps Repositories
        $this->userRepository = $this->entityManager->getRepository(Utilities::PROFILE_ENTITY_DIR . 'ZhUser');
        $this->productRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Product');
        $this->productFilesRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'ProductFiles');
        $this->productFilesMnemonicRepository = $this->entityManager->getRepository(Utilities::DEVOPS_ENTITY_DIR . 'ProductFilesMnemonic');
        $this->subreleaseProductFilesRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'SubreleaseProductFiles');
        $this->userAllocationRepository = $this->entityManager->getRepository(Utilities::DEVOPS_ENTITY_DIR . 'ZhUserAllocation');

        //PM Repositories
        $this->projectRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Project');
    }

    /**
     * @param $productId
     * @param $fileName
     * @param $relativePath
     * @param $id
     * @param null $page
     * @param null $pageSize
     * @return array
     */
    public function findProductFilesByFileNameAndRelativePath($productId, $fileName, $relativePath, $id, $page = null, $pageSize = null) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $organizationId = $this->environment->getOrganizationId();
        return $this->productFilesRepository->findProductFilesByProductAndFileNameAndRelativePath(
                   $productId, $fileName, $relativePath, $id, $organizationId, $limit, $offset
               );
    }

    /**
     * @param Project $project
     * @param $status
     * @param $name
     * @param $url
     * @param $path
     * @param $seqNo
     * @param $isMaster
     * @param $description
     * @param $oldUrl
     * @param $repositoryId
     */
    private function persistChildRepository(Project $project, $status, $name, $url, $path, $seqNo,
                                            $isMaster, $description, $oldUrl, $repositoryId) {
        $organization = $project->getOrganization();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Repository $repositoryRepository */
        $repositoryRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Repository');
        $childRepository = $repositoryRepository->findOneBy(array('project' => $project->getId(), 'url' => $oldUrl));
        if(empty($childRepository)) {
            $description = empty($repositoryId) ? $description : $repositoryRepository->find($repositoryId)->getDescription();
            $childRepository = new Repository();
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\Product $productRepository */
            $productRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Product');
            $product = $productRepository->findOneBy(array('project' => $project->getId()));
        } else {
            if($childRepository->getProduct()) {
                /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Product $product */
                $product = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Product', $childRepository->getProduct());
            } else {
                $product = null;
            }
        }

        $this->repositoryFactory->setRepositoryRecords($childRepository, $product, $project, $name, $url, $path, $seqNo, $isMaster, $description, $status);
        $this->entityManager->persist($childRepository);
    }

    /**
     * @param $repositoryId            - Id do Repositório
     * @param $productId               - Id do Produto
     * @param $projectId               - Id do Projeto
     * @param $isProjectAssociation    - Se o Produto está associado a algum projeto (True/False)
     * @param $name                    - Nome do Repositório
     * @param $url                     - Url do Repositório
     * @param $path                    - Diretório do Repositório
     * @param $seqNo                   - Sequência do Repositório
     * @param $isMaster                - Se o Repositório é o principal (Y/N)
     * @param $description             - Descrição do Repositório
     * @param $status                  - Status do Repositório
     * @throws \Exception
     *
     * Salva o repositório e atualiza o mesmo nos projetos filhos, caso o produto esteja associado a algum projeto.
     */
    public function saveRepositoryAndChildrenProjects($repositoryId, $productId, $projectId, $isProjectAssociation,
                                                     $name, $url, $path, $seqNo, $isMaster, $description, $status) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Product $product */
            $product = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Product', $productId);
            if($repositoryId) {
                $repository = $this->entityManager->find(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Repository', $repositoryId);
                $oldUrl = $repository->getUrl();
            } else {
                $repository = new Repository();
                $oldUrl = $url;
            }
            if($projectId) {
                $project = $this->entityManager->find(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Project', $projectId);
            } else {
                $project = null;
            }
            $repository = $this->repositoryFactory->setRepositoryRecords($repository, $product, $project, $name, $url, $path, $seqNo, $isMaster, $description, $status);
            $this->entityManager->persist($repository);

            if($isProjectAssociation && $project) {
                /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Project $projectRepository */
                $projectRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Project');
                $childrenProjects = $projectRepository->getProjectChildren($project->getTreeKey());

                foreach ($childrenProjects as $childProject) {
                    $this->persistChildRepository($childProject, $status, $name, $url, $path, $seqNo,
                                                  $isMaster, $description, $oldUrl, $repositoryId);
                }
            }
            $this->entityManager->flush();
            $connection->commit();
            return array('ID' => $repository->getId());
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $productId
     * @throws \Exception
     */
    public function removeProduct($productId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Product $product */
            $product = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Product', $productId);
            $releases = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Release')
                ->findBy(array('product' => $product));

            foreach($releases as $release) {
                $subreleases = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease')
                    ->findBy(array('release' => $release));
                $this->removeSubreleasesAndBranches($subreleases);

                /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release $release */
                $release->setIsActive(Utilities::IS_ACTIVE_DELETED);
                $this->entityManager->persist($release);
            }

            $product->setIsActive(Utilities::IS_ACTIVE_DELETED);
            $this->entityManager->persist($product);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $subreleases
     */
    private function removeSubreleasesAndBranches($subreleases)
    {
        foreach ($subreleases as $subrelease) {
            $branches = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Branch')
                ->findBy(array('subrelease' => $subrelease));

            foreach ($branches as $branch) {
                /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Branch $branch */
                $branch->setSubrelease(null);
                $this->entityManager->persist($branch);
            }
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subrelease */
            $subrelease->setIsActive(Utilities::IS_ACTIVE_DELETED);
            $this->entityManager->persist($subrelease);
        }
    }

    public function deactivateReleases($productId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Product $product */
            $product = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Product', $productId);
            $releases = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Release')
                ->findBy(array('product' => $product));

            foreach($releases as $release) {
                $subreleases = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease')
                    ->findBy(array('release' => $release));

                foreach($subreleases as $subrelease) {
                    /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subrelease */
                    $subrelease->setIsActive(Utilities::IS_ACTIVE_NO);
                    $this->entityManager->persist($subrelease);
                }

                /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release $release */
                $release->setIsActive(Utilities::IS_ACTIVE_NO);
                $this->entityManager->persist($release);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function findProductsByServer($serverId) {
        return $this->productRepository->getProductsByServer($serverId);
    }

    /**
     * @param $conditions
     * @return array
     */
    public function findProduct($conditions, $page = null, $pageSize = null) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filters = Utilities::createQueryFiltered($conditions);
        $organizationId = $this->environment->getOrganizationId();
        $productRegisters = $this->productRepository->getProduct($filters,$organizationId, $limit, $offset);
        foreach($productRegisters as &$product) {
            if($product['USER_ID']) {
                $product['USER_ID'] = Utilities::setSelectMultiInQuery($product['USER_ID'], true);
            }
            if($product['ZH_USER']) {
                $product['ZH_USER'] = Utilities::setSelectMultiInQuery($product['ZH_USER'], true);
            }
        }

        return $productRegisters;
    }

    /**
     * @param $id
     * @param $name
     * @param $baseUrl
     * @param $userIds
     * @param $version
     * @param $revision
     * @param $build
     * @param $description
     * @param $isActive
     * @param $projectId
     * @throws \Exception
     * @return int
     */
    public function saveProduct($id, $name, $baseUrl, $userIds, $version, $revision, $build, $description, $isActive, $projectId, $numberLtsAvailable) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            /** @var ConfigurationEntities\Product $product */
            $product = $this->productRepository->find($id);
            $project = $projectId ? $this->projectRepository->find($projectId) : null;
            $product = $this->productFactory->setProductRecords($product, $project, $name, $baseUrl, $description,
                $version, $revision, $build, $isActive, $numberLtsAvailable);
            $this->entityManager->persist($product);
            $this->entityManager->flush();

            $this->saveProductUsers($product, $userIds);
            $this->entityManager->flush();
            $connection->commit();

            return $product->getId();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param ConfigurationEntities\Product $product
     * @param $userIds
     */
    private function saveProductUsers(ConfigurationEntities\Product $product, $userIds) {
        foreach ($userIds as $userId) {
            /** @var ProfileEntities\ZhUser $user */
            $user = $this->userRepository->find($userId);
            /** @var DevOpsEntities\ZhUserAllocation $userAllocation */
            $userAllocation = $this->userAllocationRepository->findOneBy(array('product' => $product, 'user' => $user));
            if (empty($userAllocation)) {
                $userAllocation = $this->zhUserAllocationFactory->setZhUserAllocationRecords($userAllocation, $user, $product, null, null);
                $this->entityManager->persist($userAllocation);
            }
        }

        $userAllocations = $this->userAllocationRepository->findBy(array('product' => $product));
        foreach ($userAllocations as $zhUserAllocation) {
            /** @var DevOpsEntities\ZhUserAllocation $zhUserAllocation */
            if (!in_array($zhUserAllocation->getUser()->getId(), $userIds)) {
                $this->entityManager->remove($zhUserAllocation);
            }
        }
    }

    public function removeProductFiles($productFilesId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $productFiles = $this->productFilesRepository->find($productFilesId);
            $subProdFiles = $this->subreleaseProductFilesRepository->findBy(array('productFiles' => $productFiles));
            $prodFilesMno = $this->productFilesMnemonicRepository->findBy(array('productFiles' => $productFiles));
            foreach($subProdFiles as $subProdFile) {
                $this->entityManager->remove($subProdFile);
            }
            foreach($prodFilesMno as $prodFileMno) {
                $this->entityManager->remove($prodFileMno);
            }
            $this->entityManager->flush();
            $this->entityManager->remove($productFiles);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
} 