<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 29/09/2015
 * Time: 16:46
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use Factory\ProjectRelationship as ProjectRelationshipFactory;
use Factory\SubreleaseProductFiles as SubreleaseProductFilesFactory;
use Factory\Subrelease as SubreleaseFactory;
use Factory\Release as ReleaseFactory;
use Factory\Product as ProductFactory;
use Factory\ProductVersion as ProductVersionFactory;
use Factory\ProductVersionPackages as ProductVersionPackagesFactory;
use Factory\ZhUserAllocation as ZhUserAllocationFactory;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Devops\Entities\ProductVersion;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\SubreleaseProductFiles;
use Zeedhi\DevOps\Model\Devops\Entities\ProductVersionPackages;
use Zeedhi\DevOps\Util\Environment;
use Zeedhi\DevOps\Util\SocketClient;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities as PmEntities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories as PmRepositories;
use Zeedhi\DevOps\Model\Configurationmanager\Entities as ConfigurationEntities;
use Zeedhi\DevOps\Model\Configurationmanager\Repositories as ConfigurationRepositories;
use Zeedhi\DevOps\Model\Servermanager\Repositories as ServerManagerRepositories;
use Zeedhi\DevOps\Model\Devops\Repositories as DevOpsRepositories;
use Zeedhi\DevOps\Model\Devops\Entities as DevOpsEntities;
use Zeedhi\DevOps\Model\Profile\Repositories as ProfileRepositories;
use Zeedhi\DevOps\Model\Profile\Entities as ProfileEntities;
use Util\Messages;
use Teknisa\Libs\Util\Repositories\Migration as MigrationRepository;

class Subrelease {
    const FIELD_NAME_PARENT = 'PARENT';
    const TMP_PATH_DEVOPS = '/tmp/devops_test_script/';

    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var SubreleaseProductFilesFactory $subreleaseProductFiles */
    protected $subreleaseProductFilesFactory;
    /** @var SubreleaseFactory $subreleaseFactory */
    protected $subreleaseFactory;
    /** @var ReleaseFactory $releaseFactory */
    protected $releaseFactory;
    /** @var ProductFactory $productFactory */
    protected $productFactory;
    /** @var ProductVersionFactory $productVersionFactory */
    protected $productVersionFactory;
    /** @var ProductVersionPackagesFactory $productVersionPackagesFactory */
    protected $productVersionPackagesFactory;
    /** @var ZhUserAllocationFactory $zhUserAllocationFactory */
    protected $zhUserAllocationFactory;
    protected $logDirFile;
    /** @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\SubreleaseProductFiles $subreleaseProductFilesRepository */
    protected $subreleaseProductFilesRepository;
    /** @var PmRepositories\Repository   */
    protected $repositoryRepository;
    /** @var ProjectRelationshipFactory */
    private $projectRelationshipFactory;
    /** @var PmRepositories\TaskStatus $taskStatusRepository */
    protected $taskStatusRepository;
    /** @var PmRepositories\TaskBranches $taskBranchesRepository */
    protected $taskBranchesRepository;
    /** @var PmRepositories\TaskRelationship $taskRelationshipRepository */
    protected $taskRelationshipRepository;
    /** @var PmRepositories\ChangedFiles $changedFilesRepository */
    protected $changedFilesRepository;
    /** @var PmRepositories\Task $taskRepository */
    protected $taskRepository;
    /** @var PmRepositories\Project $projectRepository */
    protected $projectRepository;
    /** @var PmRepositories\ProjectRelationship $projectRelationshipRepository */
    protected $projectRelationshipRepository;
    /** @var PmRepositories\Allocation $allocationRepository */
    protected $allocationRepository;
    /** @var ConfigurationRepositories\Subrelease subreleaseRepository */
    protected $subreleaseRepository;
    /** @var ConfigurationRepositories\Release $releaseRepository */
    protected $releaseRepository;
    /** @var ConfigurationRepositories\Branch $branchRepository */
    protected $branchRepository;
    /** @var ServerManagerRepositories\ProductServer $productServerRepository */
    protected $productServerRepository;
    /** @var ProfileRepositories\ZhUser $userRepository */
    protected $userRepository;
    /** @var DevOpsRepositories\ZhUserAllocation $userAllocationRepository */
    protected $userAllocationRepository;
    /**
     * @var Environment
     */
    private $environment;
    /**
     * @var ProductFiles
     */
    public $productFilesService;

    /**
     * @param EntityManager $entityManager
     * @param Environment $environment
     * @param SubreleaseProductFilesFactory $subreleaseProductFilesFactory
     * @param SubreleaseFactory $subreleaseFactory
     * @param ReleaseFactory $releaseFactory
     * @param ProductFactory $productFactory
     * @param $logDirFile
     * @param ProductVersionFactory $productVersionFactory
     * @param ProductVersionPackagesFactory $productVersionPackagesFactory
     * @param ProjectRelationshipFactory $projectRelationshipFactory
     * @param ZhUserAllocationFactory $zhUserAllocationFactory
     * @param ProductFiles $productFilesService
     */
    public function __construct(EntityManager $entityManager,  Environment $environment, SubreleaseProductFilesFactory $subreleaseProductFilesFactory, SubreleaseFactory $subreleaseFactory,
                                ReleaseFactory $releaseFactory, ProductFactory $productFactory, $logDirFile, ProductVersionFactory $productVersionFactory,
                                ProductVersionPackagesFactory $productVersionPackagesFactory, ProjectRelationshipFactory $projectRelationshipFactory,
                                ZhUserAllocationFactory $zhUserAllocationFactory, ProductFiles $productFilesService
    ) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->subreleaseProductFilesFactory = $subreleaseProductFilesFactory;
        $this->subreleaseFactory = $subreleaseFactory;
        $this->releaseFactory = $releaseFactory;
        $this->productFactory = $productFactory;
        $this->logDirFile = $logDirFile;
        $this->productVersionFactory = $productVersionFactory;
        $this->productVersionPackagesFactory = $productVersionPackagesFactory;
        $this->projectRelationshipFactory = $projectRelationshipFactory;
        $this->zhUserAllocationFactory = $zhUserAllocationFactory;
        $this->productFilesService = $productFilesService;

        //DevOps Repositories
        $this->subreleaseRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease');
        $this->releaseRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Release');
        $this->branchRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Branch');
        $this->productServerRepository = $this->entityManager->getRepository(Utilities::SERVER_MANAGER_ENTITY_DIR . 'ProductServer');
        $this->userRepository = $this->entityManager->getRepository(Utilities::PROFILE_ENTITY_DIR . 'ZhUser');
        $this->userAllocationRepository = $this->entityManager->getRepository(Utilities::DEVOPS_ENTITY_DIR . 'ZhUserAllocation');
        $this->subreleaseProductFilesRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'SubreleaseProductFiles');

        //PM Repositories
        $this->taskRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Task');
        $this->projectRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Project');
        $this->allocationRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Allocation');
        $this->taskStatusRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'TaskStatus');
        $this->changedFilesRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'ChangedFiles');
        $this->taskBranchesRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'TaskBranches');
        $this->taskRelationshipRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'TaskRelationship');
        $this->projectRelationshipRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'ProjectRelationship');
        $this->repositoryRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Repository');
    }

    /**
     * @param $conditions
     * @param null $page
     * @param null $pageSize
     * @return array
     */
    public function findSubrelease($conditions, $page = null, $pageSize = null)
    {
        $filters = Utilities::createQueryFiltered($conditions);
        $organizationId = $this->environment->getOrganizationId();
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $subreleaseRegisters = $this->subreleaseRepository->getSubrelease($filters, $organizationId, $limit, $offset);

        foreach($subreleaseRegisters as &$subrelease) {
            if($subrelease['USER_ID']) {
                $subrelease['USER_ID'] = Utilities::setSelectMultiInQuery($subrelease['USER_ID'], true);
            }
            if($subrelease['ZH_USER']) {
                $subrelease['ZH_USER'] = Utilities::setSelectMultiInQuery($subrelease['ZH_USER'], true);
            }
        }
        InstanceProvider::getService()->getAttachment()->validateZhUser();
        return $subreleaseRegisters;
    }

    public function findTaskNotIsCompletedOfIssues($projectId) {
        $taskRepository = InstanceProvider::getRepository()->getTask();
        $branchRepository = InstanceProvider::getRepository()->getBranch();
        $branches = $branchRepository->findBranchWithRepositoryByProject($projectId);
        $branchNames = [];
        foreach ($branches as $branch) {
            $branchNames[] = $branch['BRANCH_NAME'];
        }
        $branchName = "'" . implode("', '", $branchNames) . "'";
        $taskNotIsCompletedOfIssues = $taskRepository->getTaskNotIsCompletedOfIssues($projectId);
        return $taskRepository->findTasksByBranches($branchName, $projectId,$taskNotIsCompletedOfIssues,false);
    }

    public function generateTestEnvironmentScript($productRepositories, $currentRow, $branchToHomologate, $pathToTest,
                                                  $productFiles, $scriptBeforeProductFiles, $scriptAfterProductFiles,
                                                  $repositoriesWithNewPath, $subReleaseName, $configurationFiles)
    {
        try {
            $createTestEnvironmentScriptService = InstanceProvider::getService()->getCreateTestEnvironmentScript();
            $script = $createTestEnvironmentScriptService->createScript($productRepositories, $currentRow, $branchToHomologate,
                                                                        $pathToTest, $productFiles, $scriptBeforeProductFiles,
                                                                        $scriptAfterProductFiles, $repositoriesWithNewPath,
                                                                        $subReleaseName, $configurationFiles);
            $this->manageTmpPathDevops();
            $this->createFileTestEnvironmentReadme();
            $this->createFilesTestScripts($productFiles);
            $this->createFileTestEnvironmentScript($script);
            $this->setPendingStatusSubRelease($currentRow['ID']);
            Utilities::compactToZipRecursive(self::TMP_PATH_DEVOPS . 'tozip/', self::TMP_PATH_DEVOPS . 'testEnvironmentGenerator.zip');
        } catch(\Exception $e) {
            throw $e;
        }
    }

    private function manageTmpPathDevops()
    {
        if(file_exists(self::TMP_PATH_DEVOPS)) {
            Utilities::deleteDirectory(self::TMP_PATH_DEVOPS);
        }
        mkdir(self::TMP_PATH_DEVOPS, 0775, true);
        mkdir(self::TMP_PATH_DEVOPS . 'tozip/', 0775, true);
        mkdir(self::TMP_PATH_DEVOPS . 'tozip/scripts/', 0775, true);
    }

    private function createFileTestEnvironmentScript($script)
    {
        try {
            $file = file_put_contents(self::TMP_PATH_DEVOPS . 'tozip/testEnvironment.sh', $script, FILE_APPEND);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function createFilesTestScripts($productFiles)
    {
        try {
            foreach ($productFiles as $pf) {
                file_put_contents(self::TMP_PATH_DEVOPS . '/tozip/scripts/' . $pf['FILE_NAME'], $pf['TEXT']);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function createFileTestEnvironmentReadme()
    {
        try {
            $text = InstanceProvider::getService()->getCreateTestEnvironmentScript()->getReadmeFileText();
            $file = file_put_contents(self::TMP_PATH_DEVOPS . 'tozip/README',  $text, FILE_APPEND);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function testEnvironmentScriptDownload()
    {
        $filename = self::TMP_PATH_DEVOPS . 'testEnvironmentGenerator.zip';
        if (file_exists($filename)) {
            header('Content-Type: application/zip');
            header("Content-Disposition: attachment; filename=" . "testEnvironmentGenerator.zip");
            header('Content-Length: ' . filesize($filename));
            readfile($filename);
            unlink($filename);
            Utilities::deleteDirectory(self::TMP_PATH_DEVOPS);
        } else {
            throw new \Exception('Arquivo não encontrado.');
        }
    }

    public function setPendingStatusSubRelease($subReleaseId)
    {
        /** @var ConfigurationEntities\Subrelease $subRelease */
        $subRelease = $this->subreleaseRepository->find($subReleaseId);
        $subRelease->setStatus(ConfigurationEntities\Subrelease::PENDING_STATUS);
        $this->entityManager->persist($subRelease);
        $this->entityManager->flush();
    }

    /**
     * @param $subreleaseId
     * @param $typeServer
     * @return array
     *
     * Busca os servidores relacionados ao Produto da Subrelease.
     */
    public function findServersBySubReleaseAndType($subreleaseId, $typeServer) {
       return $this->subreleaseRepository->findServersBySubReleaseIdAndType($subreleaseId, $typeServer);
    }

    /**
     * @param $productId
     * @return array
     */
    public function findTestOrPackageServersByProduct($productId, $filter = array()) {
        $filterString = Utilities::createQueryFiltered($filter);
        return $this->subreleaseRepository->findTestOrPackageServersByProduct($productId, $filterString);
    }

    /**
     * @param $subreleaseId
     * @return array
     *
     * Busca os arquivos relacionados ao Produto da Subrelease.
     */
    public function findProductFilesBySubRelease($subreleaseId) {
         return $this->subreleaseRepository->findProductFilesBySubReleaseId($subreleaseId);
    }

    /**
     * @param $productId
     * @param $subreleaseId
     * @param $typeEvent
     * @param $filter
     * @param $page
     * @param $pageSize
     * @return array
     */
    public function findSelectedProductFilesByProduct($productId, $subreleaseId, $typeEvent, $filter = false, $page = null, $pageSize = null)
    {   
        $limit   = $page * $pageSize;
        $offset  = ($page - 1) * $pageSize;
        $version = $revision = null;

        /** @var ConfigurationEntities\Subrelease $subrelease */
        $subrelease = $this->subreleaseRepository->find($subreleaseId);
        if($subrelease && $subrelease->getProductVersion()) {
            $version  = $subrelease->getProductVersion()->getVersion();
            $revision = $subrelease->getProductVersion()->getRevision();
        }

        $scripts = $this->subreleaseProductFilesRepository->findSelectedProductFilesByProductId($productId, $subreleaseId,
            $version, $revision, $filter, $limit, $offset);

        if ($typeEvent) {
            $scripts = $this->productFilesService->setMnemonicsInScriptBySubRelease($scripts, $typeEvent, $subrelease);
        }
        return $scripts;
    }

    public function validateSubReleaseServerMnemonic($subReleaseId, $typeEvent){
        /** @var ConfigurationEntities\Subrelease $subRelease */
        $subRelease = $this->subreleaseRepository->find($subReleaseId);
        return $this->productFilesService->validateSubReleaseServerMnemonic($subRelease, $typeEvent);
    }

    /**
     * @param $id - Id do Arquivo de Configuração da Subrelease
     * @throws \Exception
     *
     * Remove um registro da tabela de associação entre Product Files / Subrelease
     */
    public function removeSubreleaseProductFiles($id) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\SubreleaseProductFiles $subreleaseProductFilesRepository */
            $subreleaseProductFiles = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'SubreleaseProductFiles', $id);
            if(is_object($subreleaseProductFiles)) {
                $this->entityManager->remove($subreleaseProductFiles);
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removePathSubReleaseForController($subreleaseId){
        /** @var ConfigurationEntities\Subrelease $subrelease */
        $subrelease = $this->subreleaseRepository->find($subreleaseId);
        return $this->removePathSubRelease($subrelease);
    }

    /**
     * @param $subReleaseId
     * @return bool
     * @throws \Exception
     */
    public function checkPermissionSub($subReleaseId) {
        $currentUserId     = $this->environment->getCurrentUserId();
        $currentCdOperador = $this->environment->getCurrentCdoperador();
        $resource = InstanceProvider::getRepository()->getResource()->findOneBy(array('externalCode' => $currentCdOperador));
        if($resource) {
            $zhUser = InstanceProvider::getRepository()->getZhUser()->findOneBy(array('externalResId' => $resource->getId()));
            if($zhUser) {
                $currentUserId = $zhUser->getId();
                $this->environment->setCurrentUserId($currentUserId);
            }
        }
        $user = $this->userAllocationRepository->findOneBy(array('user' => $currentUserId, 'subrelease' => $subReleaseId));
        if (!$user && !empty($currentUserId)) {
            throw new \Exception(Messages::NO_PERMISSION_TO_FINISH_SUBRELEASE);
        }
        return true;
    }


    public function removePathSubRelease(ConfigurationEntities\Subrelease $subrelease)
    {
        try{
            /** @var ConfigurationEntities\Subrelease $subrelease */
            $response = array();
            if ($subrelease->getUrlToTest()) {
                $wsEmail = $subrelease->getProductServer()->getServer()->getWsemail();
                $path = $subrelease->getPathToTest();
                if($wsEmail && $path){
                    $socket = SocketClient::factory();
                    $response = $socket->removeDir($wsEmail, $path);
                }
            }
            return $response;
        }catch (\Exception $e){ }
    }

    /**
     * @param $subreleaseId   - Id da Subrelease
     * @param $productFilesId - Id do Arquivo de Configuração
     * @throws \Exception
     *
     * Salva um registro na tabela de associação entre Product Files / Subrelease
     */
    public function saveSubreleaseProductFiles($subreleaseId, $productFilesId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subrelease */
            $subrelease = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease', $subreleaseId);

            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\ProductFiles $server */
            $productFile = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'ProductFiles', $productFilesId);

            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\SubreleaseProductFiles $subreleaseProductFiles */
            $subreleaseProductFiles = new SubreleaseProductFiles();
            $subreleaseProductFiles = $this->subreleaseProductFilesFactory->setSubreleaseProductFilesRecords($subreleaseProductFiles, $subrelease, $productFile);

            $this->entityManager->persist($subreleaseProductFiles);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $subreleaseId   - Id da Subrelease
     * @param $productFilesId - Id do Arquivo de Configuração
     * @throws \Exception
     *
     * Salva um registro na tabela de associação entre Product Files / Subrelease
     */
    public function saveProductFileRequiredOnce($subreleaseId, $productFilesId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subrelease */
            $subrelease = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease', $subreleaseId);

            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\ProductFiles $productFile */
            $productFile = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'ProductFiles', $productFilesId);

            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\ProductFiles $productFiles */
            $productFiles = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'ProductFiles')
                ->findProductFilesByFileNameAndRelativePath($subrelease->getRelease()->getProduct()->getId(), $subreleaseId,
                                                            $productFile->getFileName(), $productFile->getRelativePath());

            if(empty($productFiles)) {
                $subreleaseProductFiles = new SubreleaseProductFiles();
            } else {
                /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\SubreleaseProductFiles $subreleaseProductFiles */
                $subreleaseProductFiles = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'SubreleaseProductFiles', $productFiles[0]['ID']);
            }

            $subreleaseProductFiles = $this->subreleaseProductFilesFactory->setSubreleaseProductFilesRecords($subreleaseProductFiles, $subrelease, $productFile);

            $this->entityManager->persist($subreleaseProductFiles);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $projectId
     * @return array
     */
    public function findTasksNotConcluded($subReleaseId) {
        $tasks = $this->taskRepository->findNotConcludedTasksBySubrelease($subReleaseId);
        $taskValidation = [];
        

        /** @var PmEntities\Task $task */
        foreach($tasks as $task) {
            if(!isset($taskValidation[$task['PROJECT_ID']])) {
                $taskValidation[$task['PROJECT_ID']] = array(
                    'PROJECT_NAME' => $task['PROJECT_NAME'],
                    'TASK'         => array()
                );
            }
            $taskValidation[$task['PROJECT_ID']]['TASK'][] = '"' . $task['NAME'] . '"';
        }
        /*
        $projects = $this->taskRepository->findNotConcludedProjectTestsBySubrelease($subReleaseId);
         foreach($projects as $project) {
            if(!isset($taskValidation[$project['PROJECT_ID']])) {
                $taskValidation[$project['PROJECT_ID']] = array(
                    'PROJECT_NAME' => $project['NAME'],
                    'VALIDATION_TYPE' => 'PROJECT'
                );
            }
        }
        */

        return !empty($taskValidation) ? [array_values($taskValidation)] : $taskValidation;
    }

    public function validateFinishSubRelease($subReleaseId) {
        return InstanceProvider::getRepository()->getTask()->findTaskWithPendentTestBySubrelease($subReleaseId);
    }

    /**
     * @param $projectId
     * @return array
     */
    public function findTaskBranchesByProject($projectId) {
        $taskBranches = $this->taskRepository->findTaskBranchesByProject($projectId);
        $branchTasks = [];
        foreach($taskBranches as $branches) {
            $branchTasks[] = $branches['NAME'];
        }
        return $branchTasks;
    }

    /**
     * @param $branches
     * @param $projectId
     * @return array
     *
     * Busca as tarefas que serão necessário gerar testes. Caso a tarefa de teste já exista e
     * não estiver concluída, a tarefa geradora não será retornada.
     */
    public function findTasksNotTestGeneratedByBranches($branches, $projectId) {
        $branches = "'" . implode("', '", $branches) . "'";
        $taskRepository = InstanceProvider::getRepository()->getTask();
        $taskNotIsCompletedOfIssues = $taskRepository->getTaskNotIsCompletedOfIssues($projectId);
        return $taskRepository->findTasksByBranches($branches, $projectId,$taskNotIsCompletedOfIssues);
    }

    /**
     * @param $branch
     * @return array
     */
    public function findConflictTaskGeneratedByBranch($branch) {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Task $taskRepository */
        $taskRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Task');
        return $taskRepository->findConflictTaskGeneratedByBranch($branch);
    }

    /**
     * @param $subreleaseId
     * @param $version
     * @param $revision
     * @param $build
     * @param $status
     * @param $isProjectAssociation
     * @param $packages
     * @param $modulesConfiguration
     * @throws \Exception
     */
    public function homologateSubRelease($subreleaseId, $version, $revision, $build, $status, $isProjectAssociation,
                                         $packages, $modulesConfiguration = '') {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $subrelease = $this->setSubReleaseRecordsFinish($subreleaseId, $version, $revision, $build, $status);
            if(!empty($version) || $version == '0') {
                $product = $subrelease->getRelease()->getProduct();
                $isUpdateVersionProduct = $this->checkUpdateVersionProduct($product, $version, $revision, $build);
                if($isUpdateVersionProduct || !$subrelease->getProductVersion()) {
                    $release = $this->setReleaseRecordsFinish($version, $revision, $build, $subrelease);
                    $this->setProductRecordsFinish($version, $revision, $build, $release);
                }
                $productVersion = $this->setProductVersionRecordsFinish($version, $revision, $build, $product);
                $this->setProductVersionPackagesRecordsFinish($productVersion, $packages, $subrelease->getProductServer());

                if(!empty($modulesConfiguration)) {
                    $this->setModulesConfiguration($product, $modulesConfiguration, $productVersion, $isUpdateVersionProduct);
                }
            }

            if($isProjectAssociation &&
              (($subrelease->getRelease()->getIsFixRelease() == 'Y') ||
               ($subrelease->getProject()->getId() != $subrelease->getRelease()->getProject()->getId()))) {
                $this->concludeProjects($subrelease);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param ConfigurationEntities\Product $product
     * @param $version
     * @param $revision
     * @param $build
     * @return bool
     */
    private function checkUpdateVersionProduct(ConfigurationEntities\Product $product, $version, $revision, $build) {
        return ($version >  $product->getVersion()) ||
               ($version == $product->getVersion() && $revision > $product->getRevision()) ||
               ($version == $product->getVersion() && $revision == $product->getRevision() && $build > $product->getBuild());
    }

    /**
     * @param $subreleaseId
     * @param $version
     * @param $revision
     * @param $build
     * @param $status
     * @return \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease
     */
    private function setSubReleaseRecordsFinish($subreleaseId, $version, $revision, $build, $status)
    {
        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subrelease */
        $subrelease = $this->entityManager->find(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease', $subreleaseId);
        $subrelease = $this->subreleaseFactory->setSubreleaseRecords($subrelease, $subrelease->getProductServer(), $subrelease->getRelease(),
            $subrelease->getProject(), $subrelease->getName(), $status, $subrelease->getDescription(),
            $subrelease->getBranchToDeploy(), $version, $revision, $build, $subrelease->getPathToTest(),
            $subrelease->getUrlToTest(), 'Y', $subrelease->getReleaseProductServerId()
        );
        $this->entityManager->persist($subrelease);
        return $subrelease;
    }

    /**
     * @param $version
     * @param $revision
     * @param $build
     * @param $subrelease
     * @return \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release
     */
    private function setReleaseRecordsFinish($version, $revision, $build, $subrelease)
    {
        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subrelease */
        $release = $subrelease->getRelease();
        $release = $this->releaseFactory->setReleaseRecords($release, $release->getName(), $release->getProduct(),
            $release->getProductServer(), $release->getUpdateVersion(), $release->getIsFixRelease(), $release->getStatus(), $version, $revision, $build, $release->getDescription()
        );
        $this->entityManager->persist($release);
        return $release;
    }

    /**
     * @param $version
     * @param $revision
     * @param $build
     * @param $release
     */
    private function setProductRecordsFinish($version, $revision, $build, $release)
    {
        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release $release */
        $product = $release->getProduct();
        $product = $this->productFactory->setProductRecords($product, $product->getProject(), $product->getName(),
            $product->getBaseUrl(), $product->getDescription(), $version, $revision, $build, $product->getIsActive(), $product->getNumberLtsAvailable()
        );
        $this->entityManager->persist($product);
    }

    /**
     * @param $version
     * @param $revision
     * @param $build
     * @param $product
     * @return \Zeedhi\DevOps\Model\Devops\Entities\ProductVersion
     */
    private function setProductVersionRecordsFinish($version, $revision, $build, $product)
    {
        $productVersion = new ProductVersion();
        $this->productVersionFactory->setProductVersionRecords($productVersion, $product, $version, $revision, $build,
            ProductVersion::IS_REVISION_NO, ProductVersion::IS_BUILD_YES);
        $this->entityManager->persist($productVersion);
        return $productVersion;
    }

    /**
     * @param $productVersion
     * @param $packages
     * @param $productServer
     */
    public function setProductVersionPackagesRecordsFinish($productVersion, $packages, $productServer) {
        /** @var \Zeedhi\DevOps\Model\Servermanager\Entities\ProductServer $productServer */
        foreach($packages as $package) {
            $productVersionPackages = new ProductVersionPackages();
            $this->productVersionPackagesFactory->setProductVersionPackagesRecords($productVersionPackages, $productVersion,
                $productServer->getServer(), $package['NAME'], $package['URL'], $package['PATH_TO_DEPLOY'], $package['FILE_NAME_NOT_PATH']);
            $this->entityManager->persist($productVersionPackages);
        }
    }

    public function concludeProjectParent(PmEntities\Project $project, PmEntities\TaskStatus $projectStatusConcluded)
    {
        $projectParent = $project->getParent();
        $arrayControlRelationshipDevOps = Utilities::arrayControlRelationshipDevOps();
        if ($projectParent &&
            $projectParent->getProjectType() &&
            !in_array($projectParent->getProjectType()->getExternaltype(), $arrayControlRelationshipDevOps)
        ) {
            /** @var PmEntities\Project[] $projectChildren */
            $projectChildren = $this->projectRepository->findBy(array('parent' => $projectParent));
            $validate = true;
            foreach ($projectChildren as $projectChild) {
                if ($projectChild->getProjectStatus() &&
                    $projectChild->getProjectStatus()->getId() != $projectStatusConcluded->getId() &&
                    $projectChild->getProjectType()->getExternaltype() != PmEntities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE
                ) {
                    $validate = false;
                }
            }
            if($validate){
                $projectParent->setProjectStatus($projectStatusConcluded);
                $this->entityManager->persist($project);
                $this->entityManager->flush();
            }
        }
    }

    /**
     * @param ConfigurationEntities\Subrelease $subrelease
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    private function concludeProjects($subrelease)
    {
        $project = $subrelease->getProject();
        $pendingSubReleases = $this->subreleaseRepository->findSubreleaseNotFinalizedByProject($subrelease->getId(), $project->getId());
        $statusToSet = empty($pendingSubReleases) ? 'C' : 'O';

        /** @var PmEntities\TaskStatus $projectStatusToSet */
        $projectStatusToSet = $this->taskStatusRepository->findOneBy(array('isProject' => 'Y', 'control' => $statusToSet));
        $project->setProjectStatus($projectStatusToSet);
        $this->entityManager->persist($project);
        $this->entityManager->flush();
        $this->concludeProjectParent($project, $projectStatusToSet);

        $projectRelationship = $this->projectRelationshipRepository->findBy(array('project' => $project, 'type' => PmEntities\ProjectRelationship::TYPE_TEST));
        /** @var PmEntities\ProjectRelationship $relationship */
        foreach ($projectRelationship as $relationship) {
            $testProject = $relationship->getProjectRelationship();
            $testProject->setProjectStatus($projectStatusToSet);
            $this->entityManager->persist($testProject);
            $this->entityManager->flush();
            $this->concludeProjectParent($project, $projectStatusToSet);
        }
    }

    /**
     * @param $fileName
     * @param $fileContents
     * @throws \Exception
     */
    public function createLogFile($fileName, $fileContents) {
        $file = $this->logDirFile . DIRECTORY_SEPARATOR . $fileName;
        try {
            file_put_contents($file, $fileContents, FILE_APPEND);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $fileName
     * @throws \Exception
     * @return string
     */
    public function getLogFileContents($fileName) {
        $file = $this->logDirFile . DIRECTORY_SEPARATOR . $fileName;
        try {
            return file_exists($file) ? [file_get_contents($file)] : [];
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $subreleaseId
     * @return array
     */
    public function findIssuesPartialCompletedBySubreleaseId($subreleaseId) {
        return $this->taskRepository->getIssuesPartialCompletedBySubrelease($subreleaseId);
    }

    /**
     * @param $subreleaseId
     * @return array
     */
    public function findRelatedIssuesNotHomologatedBySubrelease($subreleaseId) {
        return $this->taskRepository->getRelatedIssuesNotHomologatedBySubrelease($subreleaseId);
    }

    /**
     * @param $subreleaseId
     * @throws \Exception
     */
    public function removeSubrelease($subreleaseId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var ConfigurationEntities\Subrelease $subrelease */
            $subrelease = $this->subreleaseRepository->find($subreleaseId);
            $branches = $this->branchRepository->findBy(array('subrelease' => $subrelease));

            foreach($branches as $branch) {
                /** @var ConfigurationEntities\Branch $branch */
                $branch->setSubrelease(null);
                $this->entityManager->persist($branch);
            }
            $this->removePathSubRelease($subrelease);
            $subrelease->setIsActive('D');
            $this->entityManager->persist($subrelease);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function checkIfProductValuesAreFullfilled($subreleaseId) {
        $productValues = array();
        $seqno = false;
        $path = false;
        /** @var ConfigurationEntities\Subrelease $subrelease */
        $subrelease = $this->subreleaseRepository->find($subreleaseId);
        $product = $subrelease->getRelease()->getProduct();
        $baseUrl = $product->getBaseUrl();
        $baseUrl          = empty($baseUrl) ? true : false;
        /** @var PmEntities\Repository[] $repositories */
        $repositories = $this->repositoryRepository->findBy(array('product' => $product->getId(), 'status' => 'A'));
        foreach ($repositories as $repository) {
            if(empty($repository->getSeqno())){
                $seqno = true;
            }
            if(empty($repository->getPath())){
                $path = true;
            }
        }
        if($baseUrl){
            $productValues["baseUrl"] = $baseUrl;
        }
        if($path){
            $productValues["path"] = $path;
        }
        if($seqno){
            $productValues["seqno"] = $seqno;
        }
        return $productValues;
    }

    /**
     * @param $tasks
     * @throws \Exception
     */
    public function updateTasksUrlToTest($tasks) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            foreach($tasks as $task) {
                /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $taskEntity */
                $taskEntity = $this->entityManager->find(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Task', $task['ID']);
                $taskEntity->setUrlToTest($task['URL_TO_TEST']);
                $taskEntity->setTestBlocked('N');
                $this->entityManager->persist($taskEntity);
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $branchName
     * @return array
     */
    public function findTaskByBranch($branchName) {
        return $this->taskRepository->findTaskByBranch($branchName);
    }

    /**
     * @param $projectId
     * @param $fieldName
     * @return array
     */
    public function findDynamicProjects($projectId, $fieldName) {
        if($fieldName == self::FIELD_NAME_PARENT) {
            return $this->projectRepository->findTestProjectsByParent($projectId);
        } else {
            /** @var PmEntities\Project $project */
            $project = $this->projectRepository->find($projectId);
            return array( array(
                'ID' => !empty($project) ? $project->getId() : '',
                'NAME' => !empty($project) ? $project->getName() : ''
            ) );
        }
    }

    /**
     * @param $projectId
     * @return array
     */
    public function findTestTasks($projectId) {
        $testTasks = $this->taskRepository->getTestTasksByProject($projectId);
        foreach($testTasks as &$task) {
            $allocations = $this->allocationRepository->findBy(array('task' => $task['ID']));
            $task['RESOURCES'] = array();

            /** @var PmEntities\Allocation $allocation */
            foreach($allocations as $allocation) {
                $task['RESOURCES'][] = $allocation->getResource()->getId();
            }
        }

        return $testTasks;
    }

    /**
     * @param $projectId
     * @param $projectRelationshipId
     * @throws \Exception
     */
    public function saveTestProjectRelationship($projectId, $projectRelationshipId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            if($projectId != $projectRelationshipId) {
                $projRelationship = $this->projectRelationshipRepository->findOneBy(array('project'             => $projectId,
                                                                                          'projectRelationship' => $projectRelationshipId,
                                                                                          'type'                => PmEntities\ProjectRelationship::TYPE_TEST));
                if (empty($projRelationship)) {
                    /** @var PmEntities\Project $project */
                    $project = $this->projectRepository->find($projectId);
                    /** @var PmEntities\Project $projectRelationship */
                    $projectRelationship = $this->projectRepository->find($projectRelationshipId);
                    $projRelationship = $this->projectRelationshipFactory->setProjectRelationshipRecords(null, $project, $projectRelationship, PmEntities\ProjectRelationship::TYPE_TEST);
                    $this->entityManager->persist($projRelationship);
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $id
     * @param $name
     * @param $active
     * @param $releaseId
     * @param $userIds
     * @param $productServerId
     * @param $status
     * @param $version
     * @param $revision
     * @param $build
     * @param $description
     * @param $projectId
     * @param $branchToDeploy
     * @param $pathToTest
     * @param $urlToTest
     * @throws \Exception
     * @return string
     */
    public function saveSubrelease($id, $name, $active, $releaseId, $userIds, $productServerId, $status, $version,
                                   $revision, $build, $description, $projectId, $branchToDeploy, $pathToTest, $urlToTest, $releaseProductServerId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            /** @var ConfigurationEntities\Subrelease $subrelease */
            $subrelease    = $this->subreleaseRepository->find($id);
            $release       = $this->releaseRepository->find($releaseId);
            $project       = $projectId ? $this->projectRepository->find($projectId) : null;
            $productServer = $this->productServerRepository->find($productServerId);

            $subrelease = $this->subreleaseFactory->setSubreleaseRecords($subrelease, $productServer, $release, $project, $name,
                $status, $description, $branchToDeploy, $version, $revision, $build, $pathToTest, $urlToTest, $active, $releaseProductServerId);
            $this->entityManager->persist($subrelease);
            $this->entityManager->flush();

            $this->saveSubreleaseUsers($subrelease, $userIds);
            $this->entityManager->flush();
            $connection->commit();

            return $subrelease->getId();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    private function saveSubreleaseUsers(ConfigurationEntities\Subrelease $subrelease, $userIds) {
        foreach ($userIds as $userId) {
            /** @var ProfileEntities\ZhUser $user */
            $user = $this->userRepository->find($userId);
            /** @var DevOpsEntities\ZhUserAllocation $userAllocation */
            $userAllocation = $this->userAllocationRepository->findOneBy(array('subrelease' => $subrelease, 'user' => $user));
            if (empty($userAllocation)) {
                $userAllocation = $this->zhUserAllocationFactory->setZhUserAllocationRecords($userAllocation, $user, null, null, $subrelease);
                $this->entityManager->persist($userAllocation);
            }
        }

        $userAllocations = $this->userAllocationRepository->findBy(array('subrelease' => $subrelease));
        foreach ($userAllocations as $zhUserAllocation) {
            /** @var DevOpsEntities\ZhUserAllocation $zhUserAllocation */
            if (!in_array($zhUserAllocation->getUser()->getId(), $userIds)) {
                $this->entityManager->remove($zhUserAllocation);
            }
        }
    }

    /**
     * @param $subreleaseId
     * @return array
     */
    public function findBranchesAllowedExclusion($subreleaseId) {
        /** @var ConfigurationEntities\Subrelease $subrelease */
        $subrelease = $this->subreleaseRepository->find($subreleaseId);
        $project = $subrelease->getProject();
        $branchesReturn = array();
        $branchesToExclude = $this->branchRepository->findBranchWithRepositoryUrlToExcludeBySubrelease($subreleaseId, $this->environment->getCurrentOrganizationId());
        foreach($branchesToExclude as $branch) {
            $branchesReturn[] = array('BRANCH' => $branch['NAME'],
                                      'REPOSITORIES' => explode(',', $branch['URL']));
        }

        if($project &&
           $project->getProjectType() &&
           $project->getProjectType()->getExternaltype() == PmEntities\ProjectType::TYPE_PROJECT_CONTROL_ERROR) {
            /** @var PmEntities\Repository[] $repositories */
            $repositories = $this->repositoryRepository->findBy(array('project' => $project));
            $urls = array();
            foreach ($repositories as $repository) {
                $urls[] = $repository->getUrl();
            }

            $branchesReturn[] = array(
                'BRANCH' => $project->getBranchErrorMain(),
                'REPOSITORIES' => $urls
            );
        }

        return $branchesReturn;
    }

    public function setModulesConfiguration(ConfigurationEntities\Product $product, $modulesConfiguration,
                                            DevOpsEntities\ProductVersion $productVersion, $isUpdateVersionProduct) {
        $productVersion->setModulesConfiguration(trim($modulesConfiguration));
        $this->entityManager->persist($productVersion);
        if($isUpdateVersionProduct) {
            $product->setModulesConfiguration(trim($modulesConfiguration));
            $this->entityManager->persist($product);
        }
    }

    public function validateIssueHomologation($subreleaseId, $releaseId) {
        $taskProperties = InstanceProvider::getRepository()->getTaskProperties()->findOneBy(array('organization' => $this->environment->getCurrentOrganizationId()));
        $finalTaskStatusId = $taskProperties && $taskProperties->getFinalTaskStatus() ? $taskProperties->getFinalTaskStatus()->getId() : 0;
        if($subreleaseId) {
            return $this->subreleaseRepository->getAllIssueBySubReleaseToSendMail($subreleaseId, true, $finalTaskStatusId);
        } else {
            return $this->releaseRepository->getAllIssuesByReleaseToSendMail($releaseId, true, $finalTaskStatusId);
        }
    }

    public function validateSubreleasePreviousVersionsNotFinalized($subReleaseId) {
        $productVersionId = $version = $revision = null;
        /** @var ConfigurationEntities\Subrelease $subRelease */
        $subRelease = $this->subreleaseRepository->find($subReleaseId);
        $projectId  = $subRelease->getProject()->getId();
        if($subRelease->getProductVersion()) {
            $version  = $subRelease->getProductVersion()->getVersion();
            $revision = $subRelease->getProductVersion()->getRevision();
            $productVersionId = $subRelease->getProductVersion()->getId();
        }
        return $this->subreleaseRepository->findSubreleasePreviousVersionsNotFinalized($subReleaseId, $projectId, $version, $revision, $productVersionId);
    }

    public function validateSubreleasesInTestOrPendingToFinish($subReleaseId) {
        $names = ''; $validation = 'VALID'; $validationSubreleases = array();
        /** @var ConfigurationEntities\Subrelease $subRelease */
        $subRelease = $this->subreleaseRepository->find($subReleaseId);
        $productId  = $subRelease->getRelease()->getProduct()->getId();
        
        $toFinishSubreleases = $this->subreleaseRepository->findProductSubreleasesByStatus($productId, $subReleaseId, array (
            ConfigurationEntities\Subrelease::IM_HOMOLOGATION_STATUS, ConfigurationEntities\Subrelease::PACKAGE_CREATED_STATUS 
        ));
        if(!empty($toFinishSubreleases)) {
            $validation = 'TO_FINISH';
            $validationSubreleases = $toFinishSubreleases;
        } else {
            $inTestSubreleases = $this->subreleaseRepository->findProductSubreleasesByStatus($productId, $subReleaseId, array (
                ConfigurationEntities\Subrelease::IN_TEST_STATUS
            ));
            if(!empty($inTestSubreleases)) {
                $validation = 'IN_TEST';
                $validationSubreleases = $inTestSubreleases;
            }
        }
        foreach($validationSubreleases as $validationSubrelease) {
            if(!$names) {
                $names .= '<li>' . $validationSubrelease['NAME'] . ' (ID: ' . $validationSubrelease['ID'] . ') </li>';
            } else {
                $names .= '<li>' . $validationSubrelease['NAME'] . ' (ID: ' . $validationSubrelease['ID'] . ') </li> <br>';
            }
        }
        return array (
            'TYPE'  => $validation,
            'NAMES' => $names
        );
    }

    public function checkCreateBuildToCurrentVersion($subReleaseId) {
        /** @var ConfigurationEntities\Subrelease $subRelease */
        $subRelease = $this->subreleaseRepository->find($subReleaseId);
        $currentRevision = InstanceProvider::getRepository()->getProductVersion()->findCurrentRevisionVersion (
            $subRelease->getProduct()->getId())[0];
        $subReleaseCurrentRevision = $this->subreleaseRepository->findSubreleaseByProjectAndProductVersion (
            $subRelease->getProject()->getId(), $currentRevision['ID']);
        if(empty($subReleaseCurrentRevision)) {
            $currentBuild = InstanceProvider::getRepository()->getProductVersion()->findCurrentBuildFromVersion (
                $subRelease->getProduct()->getId(), InstanceProvider::getLibEnvironment()->getCurrentOrganizationId(),
                $currentRevision['VERSION'], $currentRevision['REVISION']
            )[0];
            return array (
                'VERSION'            => $currentBuild['PR_VERSION'],
                'REVISION'           => $currentBuild['PR_REVISION'],
                'BUILD'              => $currentBuild['PR_BUILD'],
                'PRODUCT_VERSION_ID' => $currentBuild['ID']
            );
        } else {
            return array();
        }
    }

    /**
     * @param $subReleaseId
     * @param $conditions
     * @return string
     */
    public function findResponsesOfIssuesAndTasks($subReleaseId, $conditions)
    {
        $filter = Utilities::createQueryFiltered($conditions);
        $organizationId = $this->environment->getOrganizationId();
        return $this->taskRepository->findResponsesOfIssuesAndTasks([$subReleaseId], $organizationId, $filter);
    }

    public function findTaskResponsesRelatedSameIssue($issueResponse)
    {
        return $this->taskRepository->findTaskResponsesRelatedSameIssue($issueResponse);
    }

    /**
     * @param $rows
     * @return array
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function saveResponsesOfIssuesAndTasks($rows)
    {
        foreach ($rows as $row) {
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $task */
            $task = $this->taskRepository->find($row['ID']);
            $task->setResponse($row['RESPONSE']);
            $this->entityManager->persist($task);
        }
        $this->entityManager->flush();
    }

    public function findProductInfo($productId)
    {
        return array (
            'REPOSITORIES_INFO' => InstanceProvider::getRepository()->getRepository()->getMigrationRepositoriesInfoWithModules($productId, 1),
            'CURRENT_USER_DATA' => array (
                'CURRENT_ORG_ID'        => $this->environment->getCurrentOrganizationId(),
                'CURRENT_OPERATOR_CODE' => $this->environment->getCurrentCdoperador()
            ),
            'MIGRATION_QUERIES' => array(
                'INSERT' => MigrationRepository::INSERT_LAST_MIG_QUERY,
                'UPDATE' => MigrationRepository::UPDATE_LAST_MIG_QUERY,
                'PENDENT_FILE' => MigrationRepository::PENDENT_FILE_MIG_QUERY
            )
        );
    }

    public function findMigrationInfo(array $migrationConfig)
    {
        foreach ($migrationConfig as &$migration) {
            $migration['MIG_FILES_CONTENT'] = array();
            $migration['MIG_PENDENT_FILES'] = isset($migration['MIG_PENDENT_FILES']) ? $migration['MIG_PENDENT_FILES'] : array();
            $pendFiles = empty($migration['MIG_PENDENT_FILES']) ? [0] : $migration['MIG_PENDENT_FILES'];
            $pendFiles = implode(',', $pendFiles);
            $scripts   = InstanceProvider::getRepository()->getRepository()->getDeployMigration($migration['MIG_DESCRIPTION'],
                $migration['MIG_LAST_FILE_NUMBER'], $pendFiles);
            foreach ($scripts as $scriptRegister) {
                $idx = 0; $notEmptyScripts = array();
                $fullScript = explode("\n", trim($scriptRegister['SCRIPT']));
                foreach ($fullScript as $script) {
                    $trimScript = trim($script);
                    if ($trimScript !== '') {
                        if ($script[0] == '/' && (!isset($script[1]) || $script[1] != '*')) {
                            $idx++;
                        } else {
                            $notEmptyScripts[$idx] = isset($notEmptyScripts[$idx]) ? $notEmptyScripts[$idx] . "\n" . $script : $script;
                        }
                    }
                }
                $notEmptyScripts = array_values($notEmptyScripts);
                $migration['MIG_FILES_CONTENT'][] = array(
                    'FILE_NAME'     => $scriptRegister['NRSCRIPT'] . '.sql',
                    'FILE_CONTENTS' => $notEmptyScripts,
                    'IS_PEND_FILE'  => $scriptRegister['IDARQPENDENTE'] == 'S'
                );
                $migration['MIG_PENDENT_FILES'] = array_filter($migration['MIG_PENDENT_FILES'], function($pendFile) use ($scriptRegister) {
                    return $pendFile != $scriptRegister['NRSCRIPT'];
                });
                $migration['MIG_PENDENT_FILES'] = array_values($migration['MIG_PENDENT_FILES']);
            }
        }
        return $migrationConfig;
    }

    /**
     * @param array $documents
     * @param $deployId
     * @param null $type
     * @return mixed
     * @throws \Exception
     */
    public function saveLog(array $documents, $subReleaseId, $type = null)
    {
        try {
            $subRelease = $this->subreleaseRepository->find($subReleaseId);
            return $this->updateLog($documents, $subRelease, $type);
        } catch (\Exception $e){
            throw $e;
        }
    }

    public function updateLog(array $documents, $subRelease, $type){
        try {
            $filesUploaded = array();
            foreach ($documents as $fileData) {
                $labelFile = empty($fileData['name']) ? $fileData['NAME'] : $fileData['name'];
                /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment $oldLogFile */
                $oldLogFile = InstanceProvider::getRepository()->getAttachment()->findOneBy(array("label" => $labelFile, "subrelease" => $subRelease));

                $typeFile = empty($fileData['type']) ? $type : $fileData['type'];
                $sizeFile = empty($fileData['size']) ? null : $fileData['size'];
                $pathFile = empty($fileData['path']) ? null : $fileData['path'];
                $deployScript = empty($fileData['SCRIPT']) ? null : $fileData['SCRIPT'];

                $attachmentFactory = InstanceProvider::getFactory()->getAttachment();

                if (is_null($oldLogFile)) {
                    $attachment = $attachmentFactory->createAttachment($typeFile, $pathFile, $labelFile, $sizeFile, $subRelease, $deployScript);
                } else {
                    if($pathFile){
                        InstanceProvider::getZhFileUpload()->deleteFile($oldLogFile->getFileName());
                    }
                    $attachment = $attachmentFactory->updateAttachmentPath($oldLogFile, $pathFile, $deployScript);
                }
                $this->entityManager->persist($attachment);
                $filesUploaded[] = $attachment->getId();
            }
            $this->entityManager->flush();
            return $filesUploaded;
        } catch (\Exception $e){
            throw $e;
        }
    }

    public function findDeployMigrations($subReleaseId, $filter)
    {
        $filterQuery = Utilities::createQueryFiltered($filter);
        return InstanceProvider::getRepository()->getAttachment()->getDeployMigrationsScriptsBySubRelease($subReleaseId, $filterQuery);
    }

    public function downloadDeployMigration($id)
    {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment $attachment */
        $attachment = InstanceProvider::getRepository()->getAttachment()->findOneBy(array("id" => $id));
        $fileDir  = Utilities::getTemporaryDir();
        $fileName = $attachment->getLabel();
        $script = mb_convert_encoding( $attachment->getDeployScript(), "ISO-8859-1");
        file_put_contents($fileDir . DIRECTORY_SEPARATOR . $fileName, $script);
        Utilities::downloadAndRemoveFile($fileDir . DIRECTORY_SEPARATOR . $fileName, $fileName, 'text/plain', true);
    }

    /**
     * @param $subReleaseId
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function setPackageCreatedStatusInSubRelease($subReleaseId)
    {
        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subRelease */
        $subRelease = $this->subreleaseRepository->find($subReleaseId);
        $subRelease->setStatus(ConfigurationEntities\Subrelease::PACKAGE_CREATED_STATUS);
        $this->entityManager->persist($subRelease);
        $this->entityManager->flush();
    }

    /**
     * @param $subReleaseId
     * @param $urlToTest
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function blockOtherSubreleaseTestTasks($subReleaseId, $urlToTest)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $subRelease = $this->subreleaseRepository->find($subReleaseId);
            $productId = $subRelease->getRelease()->getProduct()->getId();
            
            //Update actual sub-release tests
            $tasks = $this->taskRepository->findTestTasksBySubreleases(array($subReleaseId));
            foreach($tasks as $task) {
                $taskEntity = $this->taskRepository->find($task['ID']);
                $taskEntity->setUrlToTest($urlToTest);
                $taskEntity->setTestBlocked('N');
                $this->entityManager->persist($taskEntity);
            }
            
            //Block other sub-releases tests
            $inTestSubreleases = $this->subreleaseRepository->findProductSubreleasesByStatus($productId, $subReleaseId, array (
                ConfigurationEntities\Subrelease::IN_TEST_STATUS
            ));
            $ids   = Utilities::getArrayFromArrayAssociative($inTestSubreleases);
            $tasks = $this->taskRepository->findTestTasksBySubreleases($ids);
            /** @var PmEntities\TaskStatus $taskStatus */
            $taskStatus = $this->taskStatusRepository->findOneBy(array('isTask' => 'Y', 'control' => 'P'));
            foreach($tasks as $task) {
                $taskEntity = $this->taskRepository->find($task['ID']);
                $taskEntity->setTestBlocked('Y');
                $taskEntity->setTaskStatus($taskStatus);
                $this->entityManager->persist($taskEntity);
                
                $allocations = $this->allocationRepository->findBy(array('task' => $task['ID']));
                /** @var PmEntities\Allocation $allocation */
                foreach($allocations as $allocation) {
                    $allocation->setTaskStatus($taskStatus);
                    $this->entityManager->persist($allocation);
                }
            }
            
            //Update other sub-releases to pending status
            foreach($ids as $id) {
                $subReleaseToUp = $this->subreleaseRepository->find($id);
                $subReleaseToUp->setStatus(ConfigurationEntities\Subrelease::PENDING_STATUS);
                $this->entityManager->persist($subReleaseToUp);
            }
            
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
}